@@ -1,8 +1,8 @@
 {- git-annex command
  -
- - Copyright 2010-2018 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2019 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 {-# LANGUAGE CPP #-}
@@ -15,8 +15,6 @@ import qualified Remote
 import qualified Types.Backend
 import qualified Backend
 import Annex.Content
-import qualified Annex.Content.Direct as Direct
-import Annex.Direct
 import Annex.Perms
 import Annex.Link
 import Logs.Location
@@ -28,7 +26,6 @@ import Annex.NumCopies
 import Annex.UUID
 import Annex.ReplaceFile
 import Utility.DataUnits
-import Config
 import Utility.HumanTime
 import Utility.CopyFile
 import Git.FilePath
@@ -88,7 +85,7 @@ optParser desc = FsckOptions
 			))
 
 seek :: FsckOptions -> CommandSeek
-seek o = allowConcurrentOutput $ do
+seek o = startConcurrency commandStages $ do
 	from <- maybe (pure Nothing) (Just <$$> getParsed) (fsckFromOption o)
 	u <- maybe getUUID (pure . Remote.uuid) from
 	checkDeadRepo u
@@ -114,7 +111,7 @@ start from inc file key = Backend.getBackend file key >>= \case
 			Nothing -> go $ perform key file backend numcopies
 			Just r -> go $ performRemote key afile backend numcopies r
   where
-	go = runFsck inc (mkActionItem afile) key
+	go = runFsck inc (mkActionItem (key, afile)) key
 	afile = AssociatedFile (Just file)
 
 perform :: Key -> FilePath -> Backend -> NumCopies -> Annex Bool
@@ -134,7 +131,7 @@ perform key file backend numcopies = do
 		]
   where
 	afile = AssociatedFile (Just file)
-	ai = ActionItemAssociatedFile afile
+	ai = mkActionItem (key, afile)
 
 {- To fsck a remote, the content is retrieved to a tmp file,
  - and checked locally. -}
@@ -161,7 +158,7 @@ performRemote key afile backend numcopies remote =
 		, withLocalCopy localcopy $ checkBackendRemote backend key remote ai
 		, checkKeyNumCopies key afile numcopies
 		]
-	ai = ActionItemAssociatedFile afile
+	ai = mkActionItem (key, afile)
 	withtmp a = do
 		pid <- liftIO getPID
 		t <- fromRepo gitAnnexTmpObjectDir
@@ -225,19 +222,19 @@ fixLink key file = do
  - in this repository only. -}
 verifyLocationLog :: Key -> KeyStatus -> ActionItem -> Annex Bool
 verifyLocationLog key keystatus ai = do
-	direct <- isDirect
 	obj <- calcRepo $ gitAnnexLocation key
-	present <- if not direct && isKeyUnlocked keystatus
+	present <- if isKeyUnlockedThin keystatus
 		then liftIO (doesFileExist obj)
 		else inAnnex key
 	u <- getUUID
 	
 	{- Since we're checking that a key's object file is present, throw
 	 - in a permission fixup here too. -}
-	when (present && not direct) $ do
-		void $ tryIO $ if isKeyUnlocked keystatus
-			then thawContent obj
-			else freezeContent obj
+	when present $ do
+		void $ tryIO $ case keystatus of
+			KeyUnlockedThin -> thawContent obj
+			KeyLockedThin -> thawContent obj
+			_ -> freezeContent obj
 		unlessM (isContentWritePermOk obj) $
 			warning $ "** Unable to set correct write mode for " ++ obj ++ " ; perhaps you don't own that file"
 	whenM (liftIO $ doesDirectoryExist $ parentDir obj) $
@@ -251,11 +248,7 @@ verifyLocationLog key keystatus ai = do
 		whenM (annexSecureHashesOnly <$> Annex.getGitConfig) $
 			warning $ "** Despite annex.securehashesonly being set, " ++ obj ++ " has content present in the annex using an insecure " ++ decodeBS (formatKeyVariety (keyVariety key)) ++ " key"
 
-	{- In direct mode, modified files will show up as not present,
-	 - but that is expected and not something to do anything about. -}
-	if direct && not present
-		then return True
-		else verifyLocationLog' key ai present u (logChange key u)
+	verifyLocationLog' key ai present u (logChange key u)
 
 verifyLocationLogRemote :: Key -> ActionItem -> Remote -> Bool -> Annex Bool
 verifyLocationLogRemote key ai remote present =
@@ -274,7 +267,7 @@ verifyLocationLog' key ai present u updatestatus = do
 			fix InfoMissing
 			warning $
 				"** Based on the location log, " ++
-				actionItemDesc ai key ++
+				actionItemDesc ai ++
 				"\n** was expected to be present, " ++
 				"but its content is missing."
 			return False
@@ -294,7 +287,7 @@ verifyLocationLog' key ai present u updatestatus = do
 {- Verifies that all repos that are required to contain the content do,
  - checking against the location log. -}
 verifyRequiredContent :: Key -> ActionItem -> Annex Bool
-verifyRequiredContent key ai@(ActionItemAssociatedFile afile) = do
+verifyRequiredContent key ai@(ActionItemAssociatedFile afile _) = do
 	requiredlocs <- S.fromList . M.keys <$> requiredContentMap
 	if S.null requiredlocs
 		then return True
@@ -309,7 +302,7 @@ verifyRequiredContent key ai@(ActionItemAssociatedFile afile) = do
 					missingrequired <- Remote.prettyPrintUUIDs "missingrequired" missinglocs
 					warning $
 						"** Required content " ++
-						actionItemDesc ai key ++
+						actionItemDesc ai ++
 						" is missing from these repositories:\n" ++
 						missingrequired
 					return False
@@ -318,61 +311,41 @@ verifyRequiredContent _ _ = return True
 {- Verifies the associated file records. -}
 verifyAssociatedFiles :: Key -> KeyStatus -> FilePath -> Annex Bool
 verifyAssociatedFiles key keystatus file = do
-	ifM isDirect (godirect, goindirect)
+	when (isKeyUnlockedThin keystatus) $ do
+		f <- inRepo $ toTopFilePath file
+		afs <- Database.Keys.getAssociatedFiles key
+		unless (getTopFilePath f `elem` map getTopFilePath afs) $
+			Database.Keys.addAssociatedFile key f
 	return True
-  where
-	godirect = do
-		fs <- Direct.addAssociatedFile key file
-		forM_ fs $ \f -> 
-			unlessM (liftIO $ doesFileExist f) $
-				void $ Direct.removeAssociatedFile key f
-	goindirect = case keystatus of
-		KeyUnlocked -> do
-			f <- inRepo $ toTopFilePath file
-			afs <- Database.Keys.getAssociatedFiles key
-			unless (getTopFilePath f `elem` map getTopFilePath afs) $
-				Database.Keys.addAssociatedFile key f
-		_ -> return ()
 
 verifyWorkTree :: Key -> FilePath -> Annex Bool
 verifyWorkTree key file = do
-	ifM isDirect ( godirect, goindirect )
-	return True
-  where
-	{- Ensures that files whose content is available are in direct mode. -}
-	godirect = whenM (isJust <$> isAnnexLink file) $ do
-		v <- toDirectGen key file
-		case v of
-			Nothing -> noop
-			Just a -> do
-				showNote "fixing direct mode"
-				a
 	{- Make sure that a pointer file is replaced with its content,
 	 - when the content is available. -}
-	goindirect = do
-		mk <- liftIO $ isPointerFile file
-		case mk of
-			Just k | k == key -> whenM (inAnnex key) $ do
-				showNote "fixing worktree content"
-				replaceFile file $ \tmp -> do
-					mode <- liftIO $ catchMaybeIO $ fileMode <$> getFileStatus file
-					ifM (annexThin <$> Annex.getGitConfig)
-						( void $ linkFromAnnex key tmp mode
-						, do
-							obj <- calcRepo $ gitAnnexLocation key
-							void $ checkedCopyFile key obj tmp mode
-							thawContent tmp
-						)
-				Database.Keys.storeInodeCaches key [file]
-			_ -> return ()
+	mk <- liftIO $ isPointerFile file
+	case mk of
+		Just k | k == key -> whenM (inAnnex key) $ do
+			showNote "fixing worktree content"
+			replaceFile file $ \tmp -> do
+				mode <- liftIO $ catchMaybeIO $ fileMode <$> getFileStatus file
+				ifM (annexThin <$> Annex.getGitConfig)
+					( void $ linkFromAnnex key tmp mode
+					, do
+						obj <- calcRepo $ gitAnnexLocation key
+						void $ checkedCopyFile key obj tmp mode
+						thawContent tmp
+					)
+			Database.Keys.storeInodeCaches key [file]
+		_ -> return ()
+	return True
 
 {- The size of the data for a key is checked against the size encoded in
  - the key's metadata, if available.
  -
- - Not checked when a file is unlocked, or in direct mode.
+ - Not checked when a file is unlocked.
  -}
 checkKeySize :: Key -> KeyStatus -> ActionItem -> Annex Bool
-checkKeySize _ KeyUnlocked _ = return True
+checkKeySize _ KeyUnlockedThin _ = return True
 checkKeySize key _ ai = do
 	file <- calcRepo $ gitAnnexLocation key
 	ifM (liftIO $ doesFileExist file)
@@ -402,7 +375,7 @@ checkKeySizeOr bad key file ai = case keySize key of
 	badsize a b = do
 		msg <- bad key
 		warning $ concat
-			[ actionItemDesc ai key
+			[ actionItemDesc ai
 			, ": Bad file size ("
 			, compareSizes storageUnits True a b
 			, "); "
@@ -420,7 +393,7 @@ checkKeyUpgrade backend key ai (AssociatedFile (Just file)) =
 	case Types.Backend.canUpgradeKey backend of
 		Just a | a key -> do
 			warning $ concat
-				[ actionItemDesc ai key
+				[ actionItemDesc ai
 				, ": Can be upgraded to an improved key format. "
 				, "You can do so by running: git annex migrate --backend="
 				, decodeBS (formatKeyVariety (keyVariety key)) ++ " "
@@ -440,30 +413,19 @@ checkKeyUpgrade _ _ _ (AssociatedFile Nothing) =
  - thus when the user modifies the file, the object will be modified and
  - not pass the check, and we don't want to find an error in this case.
  - So, skip the check if the key is unlocked and modified.
- -
- - In direct mode this is not done if the file has clearly been modified,
- - because modification of direct mode files is allowed. It's still done
- - if the file does not appear modified, to catch disk corruption, etc.
  -}
 checkBackend :: Backend -> Key -> KeyStatus -> AssociatedFile -> Annex Bool
-checkBackend backend key keystatus afile = go =<< isDirect
-  where
-	go False = do
-		content <- calcRepo $ gitAnnexLocation key
-		ifM (pure (isKeyUnlocked keystatus) <&&> (not <$> isUnmodified key content))
-			( nocheck
-			, checkBackendOr badContent backend key content (mkActionItem afile)
-			)
-	go True = case afile of
-		AssociatedFile Nothing -> nocheck
-		AssociatedFile (Just f) -> checkdirect f
-	checkdirect file = ifM (Direct.goodContent key file)
-		( checkBackendOr' (badContentDirect file) backend key file (mkActionItem afile)
-			(Direct.goodContent key file)
-		, nocheck
+checkBackend backend key keystatus afile = do
+	content <- calcRepo $ gitAnnexLocation key
+	ifM (pure (isKeyUnlockedThin keystatus) <&&> (not <$> isUnmodified key content))
+		( nocheck
+		, checkBackendOr badContent backend key content ai
 		)
+  where
 	nocheck = return True
 
+	ai = mkActionItem (key, afile)
+
 checkBackendRemote :: Backend -> Key -> Remote -> ActionItem -> FilePath -> Annex Bool
 checkBackendRemote backend key remote ai localcopy =
 	checkBackendOr (badContentRemote remote localcopy) backend key localcopy ai
@@ -474,7 +436,7 @@ checkBackendOr bad backend key file ai =
 
 -- The postcheck action is run after the content is verified,
 -- in order to detect situations where the file is changed while being
--- verified (particularly in direct mode).
+-- verified.
 checkBackendOr' :: (Key -> Annex String) -> Backend -> Key -> FilePath -> ActionItem -> Annex Bool -> Annex Bool
 checkBackendOr' bad backend key file ai postcheck =
 	case Types.Backend.verifyKeyContent backend of
@@ -486,7 +448,7 @@ checkBackendOr' bad backend key file ai postcheck =
 					unless ok $ do
 						msg <- bad key
 						warning $ concat
-							[ actionItemDesc ai key
+							[ actionItemDesc ai
 							, ": Bad file content; "
 							, msg
 							]
@@ -545,14 +507,6 @@ badContent key = do
 	dest <- moveBad key
 	return $ "moved to " ++ dest
 
-{- Bad content is left where it is, but we touch the file, so it'll be
- - committed to a new key. -}
-badContentDirect :: FilePath -> Key -> Annex String
-badContentDirect file key = do
-	void $ liftIO $ catchMaybeIO $ touchFile file
-	logStatus key InfoMissing
-	return "left in place for you to examine"
-
 {- Bad content is dropped from the remote. We have downloaded a copy
  - from the remote to a temp file already (in some cases, it's just a
  - symlink to a file in the remote). To avoid any further data loss,
@@ -585,16 +539,12 @@ badContentRemote remote localcopy key = do
 		(_, False) -> "failed to drop from" ++ Remote.name remote
 
 runFsck :: Incremental -> ActionItem -> Key -> Annex Bool -> CommandStart
-runFsck inc ai key a = ifM (needFsck inc key)
-	( do
-		showStartKey "fsck" key ai
-		next $ do
-			ok <- a
-			when ok $
-				recordFsckTime inc key
-			next $ return ok
-	, stop
-	)
+runFsck inc ai key a = stopUnless (needFsck inc key) $
+	starting "fsck" ai $ do
+		ok <- a
+		when ok $
+			recordFsckTime inc key
+		next $ return ok
 
 {- Check if a key needs to be fscked, with support for incremental fscks. -}
 needFsck :: Incremental -> Key -> Annex Bool
@@ -700,28 +650,39 @@ withFsckDb (StartIncremental h) a = a h
 withFsckDb NonIncremental _ = noop
 withFsckDb (ScheduleIncremental _ _ i) a = withFsckDb i a
 
-data KeyStatus = KeyLocked | KeyUnlocked | KeyMissing
+data KeyStatus
+	= KeyMissing
+	| KeyPresent
+	| KeyUnlockedThin
+	-- ^ An annex.thin worktree file is hard linked to the object.
+	| KeyLockedThin
+	-- ^ The object has hard links, but the file being fscked
+	-- is not the one that hard links to it.
+	deriving (Show)
 
-isKeyUnlocked :: KeyStatus -> Bool
-isKeyUnlocked KeyUnlocked = True
-isKeyUnlocked KeyLocked = False
-isKeyUnlocked KeyMissing = False
+isKeyUnlockedThin :: KeyStatus -> Bool
+isKeyUnlockedThin KeyUnlockedThin = True
+isKeyUnlockedThin KeyLockedThin = False
+isKeyUnlockedThin KeyPresent = False
+isKeyUnlockedThin KeyMissing = False
 
 getKeyStatus :: Key -> Annex KeyStatus
-getKeyStatus key = ifM isDirect
-	( return KeyUnlocked
-	, catchDefaultIO KeyMissing $ do
-		unlocked <- not . null <$> Database.Keys.getAssociatedFiles key
-		return $ if unlocked then KeyUnlocked else KeyLocked
-	)
+getKeyStatus key = catchDefaultIO KeyMissing $ do
+	afs <- not . null <$> Database.Keys.getAssociatedFiles key
+	obj <- calcRepo $ gitAnnexLocation key
+	multilink <- ((> 1) . linkCount <$> liftIO (getFileStatus obj))
+	return $ if multilink && afs
+		then KeyUnlockedThin
+		else KeyPresent
 
 getKeyFileStatus :: Key -> FilePath -> Annex KeyStatus
 getKeyFileStatus key file = do
 	s <- getKeyStatus key
 	case s of
-		KeyLocked -> catchDefaultIO KeyLocked $
+		KeyUnlockedThin -> catchDefaultIO KeyUnlockedThin $
 			ifM (isJust <$> isAnnexLink file)
-				( return KeyLocked
-				, return KeyUnlocked
+				( return KeyLockedThin
+				, return KeyUnlockedThin
 				)
 		_ -> return s
+
