@@ -2,7 +2,7 @@
  -
  - Copyright 2014-2019 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 module Command.TestRemote where
@@ -24,7 +24,7 @@ import Utility.DataUnits
 import Utility.CopyFile
 import Types.Messages
 import Types.Export
-import Remote.Helper.Export
+import Remote.Helper.ExportImport
 import Remote.Helper.Chunked
 import Git.Types
 
@@ -66,8 +66,7 @@ seek :: TestRemoteOptions -> CommandSeek
 seek = commandAction . start 
 
 start :: TestRemoteOptions -> CommandStart
-start o = do
-	showStart' "testremote" (Just (testRemote o))
+start o = starting "testremote" (ActionItemOther (Just (testRemote o))) $ do
 	fast <- Annex.getState Annex.fast
 	r <- either giveup disableExportTree =<< Remote.byName' (testRemote o)
 	ks <- case testReadonlyFile o of
@@ -89,13 +88,13 @@ start o = do
 	exportr <- if Remote.readonly r'
 		then return Nothing
 		else exportTreeVariant r'
-	next $ perform rs unavailrs exportr ks
+	perform rs unavailrs exportr ks
   where
 	basesz = fromInteger $ sizeOption o
 
 perform :: [Remote] -> [Remote] -> Maybe Remote -> [Key] -> CommandPerform
 perform rs unavailrs exportr ks = do
-	ea <- maybe exportUnsupported Remote.exportActions exportr
+	let ea = maybe exportUnsupported Remote.exportActions exportr
 	st <- Annex.getState id
 	let tests = testGroup "Remote Tests" $ concat
 		[ [ testGroup "unavailable remote" (testUnavailable st r (Prelude.head ks)) | r <- unavailrs ]
@@ -154,6 +153,7 @@ adjustRemoteConfig r adjustconfig = do
 		(Remote.uuid r)
 		(adjustconfig (Remote.config r))
 		(Remote.gitconfig r)
+		(Remote.remoteStateHandle r)
 
 test :: Annex.AnnexState -> Remote -> Key -> [TestTree]
 test st r k = catMaybes
@@ -321,7 +321,7 @@ randKey sz = withTmpFile "randkey" $ \f h -> do
 		, inodeCache = Nothing
 		}
 	k <- fromMaybe (error "failed to generate random key")
-		<$> Backend.getKey Backend.Hash.testKeyBackend ks
+		<$> Backend.getKey Backend.Hash.testKeyBackend ks nullMeterUpdate
 	_ <- moveAnnex k f
 	return k
 
